Package: DeclareDesign
Title: Declare and Diagnose Research Designs
Version: 1.1.1
Authors@R: c(person("Graeme", "Blair", email = "graeme.blair@gmail.com", role = c("aut"), comment = c(ORCID = "0000-0001-9164-2102")),
             person("Jasper", "Cooper", email = "jaspercooper@gmail.com", role = c("aut"), comment = c(ORCID = "0000-0002-8639-3188")),
             person("Alexander", "Coppock", email = "acoppock@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5733-2386")),
             person("Macartan", "Humphreys", email = "macartan@gmail.com", role = c("aut"), comment = c(ORCID = "0000-0001-7029-2326")),
             person("Neal", "Fultz", email = "nfultz@gmail.com", role = c("aut")))
Description: Researchers can characterize and learn about the properties of
    research designs before implementation using `DeclareDesign`. Ex ante
    declaration and diagnosis of designs can help researchers clarify the 
    strengths and limitations of their designs and to improve their 
    properties, and can help readers evaluate a research strategy prior
    to implementation and without access to results. It can also make it
    easier for designs to be shared, replicated, and critiqued.
Depends: R (>= 4.1.0), randomizr (>= 0.20.0), fabricatr (>= 0.10.0),
        estimatr (>= 0.20.0)
Imports: rlang, generics, methods
License: MIT + file LICENSE
URL: https://declaredesign.org/r/declaredesign/,
        https://github.com/DeclareDesign/DeclareDesign
BugReports: https://github.com/DeclareDesign/DeclareDesign/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: testthat, knitr, rmarkdown, AER, diffobj, dplyr, data.table,
        tibble, ggplot2, future, future.apply, broom, MASS, Matching,
        betareg, biglm, gam, sf, reshape2, DesignLibrary, coin,
        marginaleffects, psych, CausalQueries, rdrobust, rdss
NeedsCompilation: no
Packaged: 2026-04-28 14:26:47 UTC; alexandercoppock
Author: Graeme Blair [aut] (ORCID: <https://orcid.org/0000-0001-9164-2102>),
  Jasper Cooper [aut] (ORCID: <https://orcid.org/0000-0002-8639-3188>),
  Alexander Coppock [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5733-2386>),
  Macartan Humphreys [aut] (ORCID:
    <https://orcid.org/0000-0001-7029-2326>),
  Neal Fultz [aut]
Maintainer: Alexander Coppock <acoppock@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 15:47:50 UTC; unix
