% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DeclareDesign-deprecated.R,
%   R/declare_estimator.R
\name{DeclareDesign-deprecated}
\alias{DeclareDesign-deprecated}
\alias{tidy_estimator}
\alias{model_handler}
\title{Deprecated functions in package \pkg{DeclareDesign}.}
\usage{
tidy_estimator(estimator_function)

model_handler(...)
}
\description{
The functions listed below are deprecated and will be defunct in
the near future. When possible, alternative functions with similar
functionality are also mentioned. Help pages for deprecated functions are
available at \code{help("-deprecated")}.
}
\section{\code{tidy_estimator}}{

For \code{tidy_estimator}, use \code{\link{label_estimator}}.
}

\section{\code{model_handler}}{

For \code{model_handler}, use \code{\link{method_handler}}.
}

\keyword{internal}
